---
title: llarp/layers/flow/flow_addr.hpp

---

# llarp/layers/flow/flow_addr.hpp



## Namespaces

| Name           |
| -------------- |
| **[llarp](Namespaces/namespacellarp.md)** <br>[crypto.hpp] |
| **[llarp::service](Namespaces/namespacellarp_1_1service.md)**  |
| **[llarp::layers](Namespaces/namespacellarp_1_1layers.md)**  |
| **[llarp::layers::flow](Namespaces/namespacellarp_1_1layers_1_1flow.md)**  |
| **[std](Namespaces/namespacestd.md)** <br>STL namespace.  |

## Classes

|                | Name           |
| -------------- | -------------- |
| class | **[llarp::layers::flow::FlowAddr](Classes/classllarp_1_1layers_1_1flow_1_1FlowAddr.md)** <br>a long form .snode or .loki address  |
| struct | **[std::hash< llarp::layers::flow::FlowAddr >](Classes/structstd_1_1hash_3_01llarp_1_1layers_1_1flow_1_1FlowAddr_01_4.md)**  |




## Source code

```cpp
#pragma once

#include <array>
#include <llarp/util/aligned.hpp>
#include <string>
#include <string_view>
#include <type_traits>

#include <oxenc/variant.h>

namespace llarp
{
  struct RouterID;
  namespace service
  {
    struct Address;
  }
}  // namespace llarp

namespace llarp::layers::flow
{

  class FlowAddr : public AlignedBuffer<32>
  {
   public:
    using array_t = AlignedBuffer<32>::Data;
    enum class Kind : uint8_t
    {
      none,

      snapp,
      snode,
    };

    FlowAddr();

    explicit FlowAddr(std::string str);

    FlowAddr(Kind k, array_t data);

    explicit FlowAddr(const std::variant<service::Address, RouterID>& arg);

    std::string
    ToString() const;

    bool
    operator==(const FlowAddr& other) const;

    Kind
    kind() const;

   private:
    static Kind
    get_kind(std::string_view str);

    Kind _kind;
  };

}  // namespace llarp::layers::flow

namespace llarp
{
  template <>
  inline constexpr bool IsToStringFormattable<llarp::layers::flow::FlowAddr> = true;

}

namespace std
{
  template <>
  struct hash<llarp::layers::flow::FlowAddr>
  {
    size_t
    operator()(const llarp::layers::flow::FlowAddr& addr) const
    {
      return std::hash<llarp::AlignedBuffer<32>>{}(addr)
          ^ (std::hash<uint8_t>{}(static_cast<uint8_t>(addr.kind())) << 3);
    }
  };
}  // namespace std
```


-------------------------------

Updated on 2023-04-21 at 20:16:27 +0000
