---
title: llarp/layers/platform/platform_stats.hpp

---

# llarp/layers/platform/platform_stats.hpp



## Namespaces

| Name           |
| -------------- |
| **[llarp](Namespaces/namespacellarp.md)** <br>[crypto.hpp] |
| **[llarp::layers](Namespaces/namespacellarp_1_1layers.md)**  |
| **[llarp::layers::flow](Namespaces/namespacellarp_1_1layers_1_1flow.md)**  |
| **[llarp::layers::platform](Namespaces/namespacellarp_1_1layers_1_1platform.md)**  |
| **[llarp::layers::platform::stats](Namespaces/namespacellarp_1_1layers_1_1platform_1_1stats.md)**  |

## Classes

|                | Name           |
| -------------- | -------------- |
| struct | **[llarp::layers::platform::stats::AddrInfo](Classes/structllarp_1_1layers_1_1platform_1_1stats_1_1AddrInfo.md)**  |
| struct | **[llarp::layers::platform::PlatformStats](Classes/structllarp_1_1layers_1_1platform_1_1PlatformStats.md)**  |




## Source code

```cpp
#pragma once

#include <cstdint>
#include <unordered_set>

#include <llarp/layers/flow/flow_addr.hpp>
#include <llarp/net/ip_range_map.hpp>

#include "addr_mapper.hpp"
#include "platform_addr.hpp"

namespace llarp::layers::flow
{
  class FlowLayer;
}

namespace llarp::layers::platform
{

  namespace stats
  {

    struct AddrInfo
    {
      AddressMapping addr;

      AddrInfo(const AddressMapping& ent, const flow::FlowLayer& flow_layer);

      constexpr bool
      is_exit() const
      {
        return flag(_mask_exit);
      }

      constexpr bool
      is_ready() const
      {
        return flag(_mask_ready) and is_authenticated();
      }

      constexpr bool
      is_authenticated() const
      {
        if (not flag(_mask_auth_required))
          return true;
        return flag(_mask_auth_ok);
      }

     private:
      static inline constexpr auto _mask_ready{1 << 0};
      static inline constexpr auto _mask_auth_required{1 << 1};
      static inline constexpr auto _mask_auth_ok{1 << 2};
      static inline constexpr auto _mask_inbound{1 << 3};
      static inline constexpr auto _mask_exit{1 << 4};

      constexpr bool
      flag(int mask) const
      {
        return _flags & mask;
      }

      int _flags{};
    };
  }  // namespace stats

  struct PlatformStats
  {
    std::vector<stats::AddrInfo> addrs;

    bool
    wants_client_exit() const;

    bool
    client_exit_ready() const;
  };
}  // namespace llarp::layers::platform
```


-------------------------------

Updated on 2023-04-21 at 20:16:27 +0000
